<?php

namespace Drupal\jsonapi_preview_tab\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\jsonapi_extras\EntityToJsonApi;
use Highlight\Highlighter;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller for json:api preview.
 *
 * @see \Drupal\jsonapi_preview_tab\Routing\RouteSubscriber
 * @see \Drupal\jsonapi_preview_tab\Plugin\Derivative\DevelLocalTask
 */
class JsonapiPreviewTabController extends ControllerBase {

  /**
   * The json:api extras entity to jsonapi service.
   *
   * @var \Drupal\jsonapi_extras\EntityToJsonApi
   */
  protected $entityToJsonApi;

  public function __construct(EntityToJsonApi $entity_to_jsonapi) {
    $this->entityToJsonApi = $entity_to_jsonapi;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      // Load the service required to construct this class.
      $container->get('jsonapi_extras.entity.to_jsonapi')
    );
  }

  /**
   * Builds the response.
   */
  public function build(RouteMatchInterface $route_match) {
    // Get the route parameters.
    $route_parameters = $route_match->getParameters()->all();
    // Get the entity from the first item in the array.
    $entity = reset($route_parameters);

    if (!$entity instanceof EntityInterface) {
      return NULL;
    }

    $build['#title'] = $this->t('JSON:API Preview for %title', [
      '%title' => $entity->label(),
    ]);

    // Convert the current node to a json:api response.
    $json = $this->entityToJsonApi->serialize($entity);
    // Decode and then re encode, in order to pretty print the json output.
    $json_re_encoded = json_encode(json_decode($json), JSON_UNESCAPED_SLASHES | JSON_PRETTY_PRINT);
    // Indent by 2 spaces, instead of the default 4.
    $json_two_spaced = preg_replace('/^(  +?)\\1(?=[^ ])/m', '$1', $json_re_encoded);
    // Run our json data through the php highlighter for syntax highlighting.
    $highlighter = new Highlighter();
    $json_highlighted = $highlighter->highlight('json', $json_two_spaced);

    $build['preview'] = [
      '#type' => 'inline_template',
      '#template' => '<pre class="jsonapi-preview-tab"><code class="hljs json">{{ json | raw }}</code></pre>',
      '#context' => [
        'json' => $json_highlighted->value,
      ],
      '#attached' => [
        'library' => [
          'jsonapi_preview_tab/jsonapi-preview-tab',
        ],
      ],
    ];

    return $build;
  }

}
