<?php

namespace Drupal\jsonapi_preview_tab\Routing;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\RouteSubscriberBase;
use Drupal\Core\Routing\RoutingEvents;
use Symfony\Component\Routing\Route;
use Symfony\Component\Routing\RouteCollection;

/**
 * Subscriber for JSON:API preview tab routes.
 */
class RouteSubscriber extends RouteSubscriberBase {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new RouteSubscriber object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_manager) {
    $this->entityTypeManager = $entity_manager;
  }

  /**
   * {@inheritdoc}
   */
  protected function alterRoutes(RouteCollection $collection) {
    $allowed_entity_types = [
      'media',
      'menu_link_content',
      'node',
      'taxonomy_term',
    ];

    foreach ($this->entityTypeManager->getDefinitions() as $entity_type_id => $entity_type) {
      if (in_array($entity_type_id, $allowed_entity_types)) {
        $jsonapi_preview_link = $entity_type->getLinkTemplate('jsonapi-preview');

        $route = (new Route($jsonapi_preview_link))
          ->addDefaults([
            '_controller' => '\Drupal\jsonapi_preview_tab\Controller\JsonapiPreviewTabController::build',
            '_title' => 'JSON:API Preview',
          ])
          ->addRequirements([
            '_permission' => 'access jsonapi preview tab',
          ])
          ->setOption('_admin_route', TRUE)
          ->setOption('parameters', [
            $entity_type_id => ['type' => 'entity:' . $entity_type_id],
          ]);

        $collection->add("entity.$entity_type_id.jsonapi_preview", $route);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events = parent::getSubscribedEvents();
    $events[RoutingEvents::ALTER] = 'onAlterRoutes';
    return $events;
  }

}
